@@ -16,6 +16,15 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
  */
+/* Modified by Mike Ellison February 17th, 2002 - win32 port
+
+   Just #define to using native win32 functions, since VC++ at least doesn't
+   seem to have the strncasecmp() function.  */
+
+#ifdef _MSC_VER
+#define strcasecmp  _stricmp
+#define strncasecmp _strnicmp
+#endif
 
 #include <ctype.h>
 #include <stdio.h>
@@ -24,11 +33,11 @@
 #include "tokens.h"
 #include "report.h"
 
-lexer_t     ratslexer;
-Hash        database = (Hash)NULL;
-Hash	    defaultdb = (Hash)NULL;
-char *      current_file;
-stack_t *   current_frame = (stack_t *)NULL;
+lexer_t         ratslexer;
+Hash            database = (Hash)NULL;
+Hash	        defaultdb = (Hash)NULL;
+char *          current_file;
+rats_stack_t *  current_frame = (rats_stack_t *)NULL;
 
 static int                  depths[DEPTH_COUNT];
 static int                  ungotten_token  = -1;
@@ -150,8 +159,6 @@ int get_token(void)
     return token;
 }
 
-
-   
 static
 void scan_tokens(processorfn_t fn, void *arg)
 {
@@ -197,44 +204,39 @@ void scan_tokens(processorfn_t fn, void *arg)
 static
 int gobble_backtick(int token, void *arg)
 {
-
     if (token == '`')
-      return 0;
+        return 0;
     return 1;
 }
 
 static void
 analyze_backticks(void)
 {
-  int myline = *ratslexer.lex_lineno;
+    int myline = *ratslexer.lex_lineno;
 
-  if (ratslexer.lang == LANG_C)  
-  {
-    return;
-  }
+    if (ratslexer.lang == LANG_C)  
+        return;
   
-  /* we're just gobbling up whatever is in the backticks and
-     ignoring them for now.
-  */
+    /* we're just gobbling up whatever is in the backticks and
+     * ignoring them for now.
+     */
 
-  scan_tokens(gobble_backtick, NULL);
-
-  switch(ratslexer.lang)  {
-
-    case LANG_PYTHON:
-                        log_pythonbacktick(myline, Medium);
-                        break;
-    case LANG_PHP:    
-                        log_phpbacktick(myline, Medium);
-                        break;
-    case LANG_PERL:
-                        log_perlbacktick(myline, Medium);
-                        break;
-  }
+    scan_tokens(gobble_backtick, NULL);
 
+    switch (ratslexer.lang)
+    {
+        case LANG_PYTHON:
+            log_pythonbacktick(myline, Medium);
+            break;
+        case LANG_PHP:    
+            log_phpbacktick(myline, Medium);
+            break;
+        case LANG_PERL:
+            log_perlbacktick(myline, Medium);
+            break;
+    }
 }
 
-
 static
 int check_buffer(int token, void *arg)
 {
@@ -278,7 +280,7 @@ void analyze_variable(void)
     /* If we're processing arguments right now, we don't want to check for
      * stack variables because they're not really stack variables
      */
-    if (current_frame != (stack_t *)NULL && current_frame->next != (stack_t *)NULL)
+    if (current_frame != (rats_stack_t *)NULL && current_frame->next != (rats_stack_t *)NULL)
         return;
     if (depths[DEPTH_PARENTHESIS] || depths[DEPTH_BRACKET])
         return;
@@ -295,7 +297,7 @@ int check_format_string(char *fmt, int format_arg)
 {
     int     arg = format_arg + 1;
     char *  c;
-
+    
     for (c = strchr(fmt, '%');  c != (char *)NULL;  c = strchr((fmt = c + 1), '%'))
     {
         int done = 0, precision = 0;
@@ -388,6 +390,70 @@ int check_format_string(char *fmt, int format_arg)
     return 0;
 }
 
+static
+int check_scan_format_string(char *fmt, int format_arg)
+{
+    int     arg = format_arg + 1;
+    char *  c;
+    
+    for (c = strchr(fmt, '%');  c != (char *)NULL;  c = strchr((fmt = c + 1), '%'))
+    {
+        if (*c == '*')
+        {
+            arg--;
+            c++;
+        }
+
+        switch (*c)
+        {
+            case 'a':
+            case 'L':
+            case 'q':
+                c++;
+                break;
+            case 'h':
+                if (*++c == 'h')
+                    c++;
+                break;
+
+            case 'l':
+                if (*++c == 'l')
+                    c++;
+                break;
+        }
+
+        if (format_arg == -1)
+        {
+            if (*c == 's')
+                return 1;
+        }
+        else
+        {
+            switch (*c)
+            {
+                case 's':
+                    if (!get_argument(arg)->is_constant)
+                        return 1;
+                    /* FALL THROUGH */
+                
+                case 'c':
+                case 'd': case 'D':
+                case 'i':
+                case 'o': case 'u':
+                case 'x': case 'X':
+                case 'e': case 'E':
+                case 'f': case 'F':
+                case 'g': case 'G':
+                case 'a': case 'A':
+                case 'p': case 'n':
+                    arg++;
+                    break;
+            }
+        }
+    }
+    return 0;
+}
+
 static
 int check_argument(int token, void *arg)
 {
@@ -479,33 +545,28 @@ argument_t *get_argument(int number)
     return arg;
 }
 
-
 static
 Vuln_t *locate_identifier(char *identifier)
 {
-
-    Vuln_t *data = NULL;
+    Vuln_t *    data = NULL;
     
     if (ratslexer.langhash) 
-        data = HashGet(ratslexer.langhash, identifier);
+        data = (Vuln_t *)HashGet(ratslexer.langhash, identifier);
     if ((data == NULL) && defaultdb)
-    {
-        data = HashGet(defaultdb, identifier);
-    }
+        data = (Vuln_t *)HashGet(defaultdb, identifier);
     return data;
 }
 
-
 static
 int push_identifier(char *identifier, int lineno)
 {
-    Vuln_t *    data;
-    stack_t *   frame;
+    Vuln_t *        data;
+    rats_stack_t *  frame;
 
     if (database == NULL || (data = locate_identifier(identifier)) == NULL)
         return 0;
 
-    frame = (stack_t *)malloc(sizeof(stack_t));
+    frame = (rats_stack_t *)malloc(sizeof(rats_stack_t));
     frame->identifier = identifier;
     frame->data       = data;
     frame->lineno     = lineno;
@@ -520,7 +581,7 @@ int push_identifier(char *identifier, int lineno)
 static
 void pop_identifier(void)
 {
-    stack_t *       frame;
+    rats_stack_t *  frame;
     argument_t *    arg;
     argument_t *    next;
 
@@ -614,10 +675,21 @@ void analyze_identifier(void)
             {
                 if (!arg->is_constant)
                     log_vulnerability(BOProblem, data->BOProblem->Severity);
+            
+                if (arg->is_constant && arg->contains_ps)
+                {
+                    if(data->BOProblem->Scan)
+                    {
+                        if (check_scan_format_string(arg->yytext, data->BOProblem->FormatArg))
+                            log_vulnerability(BOProblem, data->BOProblem->Severity);
+                    }
+                    else
+                    {
+                        if (check_format_string(arg->yytext, data->BOProblem->FormatArg))
+                            log_vulnerability(BOProblem, data->BOProblem->Severity);
+                    }
+                }
             }
-            if (arg->is_constant && arg->contains_ps)
-                if (check_format_string(arg->yytext, data->BOProblem->FormatArg))
-                    log_vulnerability(BOProblem, data->BOProblem->Severity);
         }
         if (data->BOProblem->SrcBufArg > 0)
             if ((arg = get_argument(data->BOProblem->SrcBufArg)) != (argument_t *)NULL)
@@ -628,11 +700,10 @@ void analyze_identifier(void)
     if (data->InputProblem != (InputProblem_t *)NULL)
     {
         analyzed++;
-        if ((arg = get_argument(data->InputProblem->Arg)) != (argument_t *)NULL){
-            if (!arg->is_constant)  {
- 
+        if ((arg = get_argument(data->InputProblem->Arg)) != (argument_t *)NULL)
+        {
+            if (!arg->is_constant)
                 log_vulnerability(InputProblem, data->InputProblem->Severity);
-            } 
         }
     }
 
@@ -720,51 +791,44 @@ void process_toctou(void)
     toctou_count = 0;
 }
 
-
 void setup_perl(FILE *fd)
 {
-
-  yyperlin = fd;
-  ratslexer.yytext = &yyperltext;
-  ratslexer.yyin =  fd;
-  ratslexer.yylex = yyperllex;
-  ratslexer.yycomment = &yyperlcomment;
-  ratslexer.lex_lineno = &perllex_lineno;
-  ratslexer.langhash = (Hash)HashGet(database, "perl");
-  ratslexer.lang = LANG_PERL;
+    yyperlin = fd;
+    ratslexer.yytext = &yyperltext;
+    ratslexer.yyin =  fd;
+    ratslexer.yylex = yyperllex;
+    ratslexer.yycomment = &yyperlcomment;
+    ratslexer.lex_lineno = &perllex_lineno;
+    ratslexer.langhash = (Hash)HashGet(database, "perl");
+    ratslexer.lang = LANG_PERL;
 }
 
-
 void setup_python(FILE *fd)
 {
-
-  yypin = fd;
-  ratslexer.yytext = &yyptext;
-  ratslexer.yyin =  fd;
-  ratslexer.yylex = yyplex;
-  ratslexer.yycomment = &yypcomment;
-  ratslexer.lex_lineno = &plex_lineno;
-  ratslexer.langhash = (Hash)HashGet(database, "python");
-  ratslexer.lang = LANG_PYTHON;
+    yypin = fd;
+    ratslexer.yytext = &yyptext;
+    ratslexer.yyin =  fd;
+    ratslexer.yylex = yyplex;
+    ratslexer.yycomment = &yypcomment;
+    ratslexer.lex_lineno = &plex_lineno;
+    ratslexer.langhash = (Hash)HashGet(database, "python");
+    ratslexer.lang = LANG_PYTHON;
 }
 
 void setup_c(FILE *fd)
 {
-
-  yycin = fd;
-  ratslexer.yytext = &yyctext;
-  ratslexer.yyin = fd;
-  ratslexer.yylex = yyclex;
-  ratslexer.yycomment = &yyccomment;
-  ratslexer.lex_lineno = &clex_lineno;
-  ratslexer.langhash = (Hash)HashGet(database, "c");
-  ratslexer.lang = LANG_C;
+    yycin = fd;
+    ratslexer.yytext = &yyctext;
+    ratslexer.yyin = fd;
+    ratslexer.yylex = yyclex;
+    ratslexer.yycomment = &yyccomment;
+    ratslexer.lex_lineno = &clex_lineno;
+    ratslexer.langhash = (Hash)HashGet(database, "c");
+    ratslexer.lang = LANG_C;
 }
 
-
 void setup_php(FILE *fd)
 {
-
     yyphpin = fd;
     ratslexer.yytext = &yyphptext;
     ratslexer.yyin = fd;
@@ -775,38 +839,35 @@ void setup_php(FILE *fd)
     ratslexer.lang = LANG_PHP;
 }
 
-
 void determine_language(char *filename, FILE *fd, int forcelang)
 {
+    char *  dot;
 
-  char *dot;
-
-
-  dot = strrchr(filename, '.');
+    dot = strrchr(filename, '.');
   
-  if (forcelang)  {
-      if (forcelang == LANG_PYTHON)
-          setup_python(fd);
-      else if (forcelang == LANG_C)
-          setup_c(fd);
-      else if (forcelang == LANG_PERL)
-          setup_perl(fd);
-      return;
-  }
-  if (!dot)
-  {
-    setup_c(fd);
-    return;
-  }    
-  if (!strcmp(dot, ".py") || !strcmp(dot, ".PY"))  {
-    setup_python(fd);
-  } else if (!strcmp(dot, ".pl") || !strcmp(dot, ".pm"))  {
-    setup_perl(fd);
-  } else if (!strcmp(dot, ".php"))  {
-    setup_php(fd);
-  } else {
-    setup_c(fd);
-  }
+    if (forcelang)
+    {
+        if (forcelang == LANG_PYTHON)
+            setup_python(fd);
+        else if (forcelang == LANG_C)
+            setup_c(fd);
+        else if (forcelang == LANG_PERL)
+            setup_perl(fd);
+        return;
+    }
+    if (!dot)
+    {
+        setup_c(fd);
+        return;
+    }
+    if (!strcasecmp(dot, ".py"))
+        setup_python(fd);
+    else if (!strcasecmp(dot, ".pl") || !strcasecmp(dot, ".pm"))
+        setup_perl(fd);
+    else if (!strcasecmp(dot, ".php"))
+        setup_php(fd);
+    else
+        setup_c(fd);
 }
 
 void process_file(char *filename, FILE *fd, int forcelang)
@@ -858,9 +919,7 @@ void analyze_comment(void)
 
     for (c += 6;  *c && isspace(*c);  c++);
     if (!*c || (*c != '$' && *c != '_' && !isalpha(*c)))
-    {
         ign = new_ignore(*ratslexer.lex_lineno, (char *)NULL);
-    }
     else
     {
         char *  p;
