@@ -22,53 +22,50 @@
 #include <stdio.h>
 #include "hash.h"
 
-Hash HashInit()  {
-    
+Hash HashInit()
+{
     return (Hash)hash_create(515, HashCompare, NULL);
 }
 
-
-
-int HashCompare(const void *firstitem, const void *seconditem)  {
-    
+int HashCompare(const void *firstitem, const void *seconditem)
+{
     return strcmp((char *)firstitem, (char *)seconditem);
-
 }
 
+int HashInsert(Hash myhash, void *item, char *name)
+{
+    hnode_t *   mynode = (hnode_t *)NULL;
+    hnode_t *   hisnode = (hnode_t *)NULL;
 
-int HashInsert(Hash myhash, void *item, char *name)  {
-
-
-    hnode_t *mynode = NULL;
-    hnode_t *hisnode = NULL;
-    if(myhash == NULL)  {
+    if (myhash == NULL)
+    {
         fprintf(stderr, "NULL Hash object passed to HashInsert\n");
         return 0;
     }
  
     mynode = hnode_create(item);
-    if(mynode == NULL)  {
+    if (mynode == NULL)
+    {
         fprintf(stderr, "Node creation failed in HashInsert\n");
         return 0;
     }
 
-    if((hisnode = hash_lookup(myhash, name)) != NULL)  {
+    if ((hisnode = hash_lookup(myhash, name)) != NULL)
+    {
         fprintf(stderr, "Function %s has multiple entries in database.  New entry will be used\n", name);
-	hash_delete(myhash,hisnode);
+	hash_delete(myhash, hisnode);
     }
     hash_insert(myhash, mynode, name);
     return 1;
 }
 
-void *HashGet(Hash myhash, char *name)  {
+void *HashGet(Hash myhash, char *name)
+{
+    hnode_t *   mynode = (hnode_t *)NULL;
+    void *      mydata = (void *)NULL;
 
-  
-    hnode_t *mynode = NULL;
-    void *mydata = NULL;
     mynode = hash_lookup(myhash, name);
-    if(mynode == NULL)  {
-        return NULL;
-    }
-    mydata = hnode_get(mynode);
+    if (mynode != (hnode_t *)NULL)
+        mydata = hnode_get(mynode);
     return mydata;
 }
